# file system tool tool :: build 46/seagull

# fonctionnalit cosmetique
$Host.UI.RawUI.BackgroundColor = ($bckgrnd = 'Black')
if ($psversiontable.PSversion.build -ne 17763) {
    #do not manipulate the window size in 1809 because MS broke powershell for that build
    [console]::WindowWidth=110
    [console]::BufferWidth=[console]::WindowWidth
    [console]::WindowHeight=30
}
clear

# boilerplate
$varProductName="Opportunity Audit"
$host.ui.RawUI.WindowTitle = "Datto RMM " + $varProductName + " Output Summary Tool"
Function EpochTime {[int][double]::Parse((Get-Date -UFormat %s))}
$varEpoch=$(epochTime)
$varScriptPath = split-path -parent $MyInvocation.MyCommand.Definition

# if we're in MTA, forcibly relaunch PS in single-thread apartment mode (fileOpenDialogue/PS2 fix)
if ([Threading.Thread]::CurrentThread.ApartmentState -match 'MTA') {
    $newProcess = new-object System.Diagnostics.ProcessStartInfo "PowerShell";
    $newProcess.Arguments = "-STA " + "-executionpolicy bypass &'" + $script:MyInvocation.MyCommand.Path + "'"
    [System.Diagnostics.Process]::Start($newProcess);
    exit # restart the shell in the proper mode
}

# function zone ---------------------------------------------------------------------------------------------------------------------------

Function Get-FileName($initialDirectory) {
    [System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms") | Out-Null
    
    $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
    $OpenFileDialog.initialDirectory = $initialDirectory
    $OpenFileDialog.filter = "$varProductName Results (*.txt)| *.txt"
    $OpenFileDialog.ShowDialog() | Out-Null
    $OpenFileDialog.filename
}

function explainStdout {
    write-host "1: Run the `"Datto Backup $varProductName`" Component on one or multiple devices concurrently as a Quick"
    write-host "   Job from the Datto RMM Web Portal."
    write-host "2: Keep the page showing `"Scheduled`", `"Running`" and `"Succeeded`" open in a tab until all devices are"
    write-host "   reporting as `"Succeeded`"; ideally none will report as `"Failed`"."
    write-host "3: Using the checkbox in the uppermost left corner of the table, select all devices."
    write-host "4: Mouse over `'Actions`' and click `'Download Selected Standard Output/Error`'."
    write-host "5: Download the Stdout contents as a TXT file and then feed that file into this script."
}

function softwareLoop ($varArray,$varLineNumber) {
    foreach ($iteration in $varArray) {
        [Console]::SetCursorPosition(30,$varLineNumber)
        $varIterationSearch=@($varFileContents | Select-String -Pattern "$iteration").count
        if ($varIterationSearch -gt 0) {
            write-host "$varIterationSearch/$varRecordQty devices" -ForegroundColor Yellow
        } else {
            write-host "No devices" -ForegroundColor DarkGray
        }
        $varLineNumber++
    }
}

function fileLoop ($varCategory,$varCategoryShorthand) {
    $varSearchResults=$varFileContents | select-string -Pattern "$varCategory"
    foreach ($iteration in $varSearchResults) {
        $iteration=((($iteration -as [string]) -split 'Found:')[-1] -replace '\.','') -as [int]
        if ($iteration -gt 0) {
            $varCurrentSum=(Get-Variable -Name varTotal$varCategoryShorthand).Value + $iteration
            Set-Variable -Name varTotal$varCategoryShorthand -Value $varCurrentSum -Scope Script
        }
    }
}

function get-ConsoleAsRTF {
	#get-ConsoleAsRTF by Vladimir Averkin; fixed by Jesse Brown
    #use as Get-ConsoleAsRtf | out-file 'whatever.rtf' -encoding ascii
	# Check the host name and exit if the host is not the Windows PowerShell console host.
    if ($host.Name -ne 'ConsoleHost')
    {
      write-host -ForegroundColor Red "This script runs only in the console host. You cannot run this script in $($host.Name)."
      exit -1
    }

    # Maps console color name to RTF color index.
    # The index of \cf is referencing the color definition in RTF color table.
    #
    function Get-RtfColorIndex ([string]$color) {
      switch ($color)
      {
        'Black' { $index = 17 }
        'DarkBlue' { $index = 2 }
        'DarkGreen' { $index = 3 }
        'DarkCyan' { $index = 4 }
        'DarkRed' { $index = 5 }
        'DarkMagenta' { $index = 6 }
        'DarkYellow' { $index = 7 }
        'Gray' { $index = 8 }
        'DarkGray' { $index = 9 }
        'Blue' { $index = 10 }
        'Green' { $index = 11 }
        'Cyan' { $index = 12 }
        'Red' { $index = 13 }
        'Magenta' { $index = 14 }
        'Yellow' { $index = 13 }
        'White' { $index = 16 }
        default
        {
          $index = 0
        }
      }
      return $index
    }

    # Create RTF block from text using named console colors.
    #
    function Append-RtfBlock ($text)
    {
      $foreColorIndex = Get-RtfColorIndex $currentForegroundColor
      $null = $rtfBuilder.Append("{\cf$foreColorIndex")

      # You can also add \ab* tag here if you want a bold font in the output.

      $backColorIndex = Get-RtfColorIndex $currentBackgroundColor
      $null = $rtfBuilder.Append("\chshdng0\chcbpat$backColorIndex")

      $text = $blockBuilder.ToString()
      $null = $rtfBuilder.Append(" $text}")
    }

    # Append line break to RTF builder
    #
    function Append-Break
    {
      $backColorIndex = Get-RtfColorIndex $currentBackgroundColor
      $null = $rtfBuilder.Append("\shading0\cbpat$backColorIndex\par`r`n")
    }

    # Initialize the RTF string builder.
    $rtfBuilder = new-object system.text.stringbuilder

    # Set the desired font
    $fontName = 'Lucida Console'
    # Append RTF header
    $null = $rtfBuilder.Append("{\rtf1\fbidis\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fnil\fcharset0 $fontName;}}")
    $null = $rtfBuilder.Append("`r`n")
    # Append RTF color table which will contain all Powershell console colors.
    $null = $rtfBuilder.Append('{\colortbl;red0\green0\blue128;\red0\green128\blue0;\red0\green128\blue128;\red128\green0\blue0;\red1\green36\blue86;\red238\green237\blue240;\red192\green192\blue192;\red128\green128\blue128;\red0\green0\blue255;\red0\green255\blue0;\red0\green255\blue255;\red255\green0\blue0;\red255\green0\blue255;\red255\green255\blue0;\red255\green255\blue255;\red0\green0\blue0;}')
    $null = $rtfBuilder.Append("`r`n")
    # Append RTF document settings.
    $null = $rtfBuilder.Append('\viewkind4\uc1\pard\ltrpar\f0\fs18 ')
 
    # Grab the console screen buffer contents using the Host console API.
    $bufferWidth = $host.ui.rawui.BufferSize.Width
    $bufferHeight = $host.ui.rawui.CursorPosition.Y
    $rec = new-object System.Management.Automation.Host.Rectangle 0,0,($bufferWidth - 1),$bufferHeight
    $buffer = $host.ui.rawui.GetBufferContents($rec)

    # Iterate through the lines in the console buffer.
    for($i = 0; $i -lt $bufferHeight; $i++)
    {
      $blockBuilder = new-object system.text.stringbuilder

      # Track the colors to identify spans of text with the same formatting.
      $currentForegroundColor = $buffer[$i, 0].Foregroundcolor
      $currentBackgroundColor = $buffer[$i, 0].Backgroundcolor

      for($j = 0; $j -lt $bufferWidth; $j++)
      {
        $cell = $buffer[$i,$j]

        # If the colors change, generate an RTF span and append it to the RTF string builder.
        if (($cell.ForegroundColor -ne $currentForegroundColor) -or ($cell.BackgroundColor -ne $currentBackgroundColor))
        {
          Append-RtfBlock

          # Reset the block builder and colors.
          $blockBuilder = new-object system.text.stringbuilder
          $currentForegroundColor = $cell.Foregroundcolor
          $currentBackgroundColor = $cell.Backgroundcolor
        }

        # Substitute characters which have special meaning in RTF.
        switch ($cell.Character)
        {
          "`t" { $rtfChar = '\tab' }
          '\' { $rtfChar = '\\' }
          '{' { $rtfChar = '\{' }
          '}' { $rtfChar = '\}' }
          default
          {
            $rtfChar = $cell.Character
          }
        }

        $null = $blockBuilder.Append($rtfChar)
      }

      Append-RtfBlock
      Append-Break
    }

    # Append RTF ending brace.
    $null = $rtfBuilder.Append('}')

    return $rtfBuilder.ToString() 
}

# script opening --------------------------------------------------------------------------------------------------------------------------

clear-host
# open
write-host "Datto RMM $varProductName | Output Summary Tool"
write-host "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-" -ForegroundColor DarkGray
write-host "`r"

# make sure the script isn't being run as a component
if ($env:cs_profile_name) {
    write-host "This script is being run as a component through Datto RMM."
    write-host "This is not how the script needs to be run."
    write-host "Please execute the script as it was received."
    exit 1
}

# intro to select a file
write-host "Please select the logfile to parse. This should be saved straight from the RMM Web Portal."
write-host "If you don't have a logfile, cancel the prompt for instructions on obtaining one."
write-host "Press ENTER to proceed" -ForegroundColor Cyan

# remind the user
[Console]::SetCursorPosition(0,20)
write-host "Your records will be saved as a text file automatically." -ForegroundColor Green
[Console]::SetCursorPosition(0,6)

# select a file
cmd /c pause 2>&1>$null
$varFile = Get-Filename

while (!$varFile) {
    write-host `r
    write-host "Error: No logfile selected."
    write-host "---------------------------"
    explainStdout
    write-host `r
    write-host "Press ENTER to select a new file, or close the window to try again later." -ForegroundColor Cyan
    cmd /c pause 2>&1>$null
    $varFile = Get-Filename   
}

# condition the filename
$varFileName=$varFile.split('\\')[-1]
if (($varFileName -as [string]).Length -gt 40) {
    $varFileName=$varFileName.SubString(0,40) + '[...].txt'
}

write-host `r
write-host "File loaded: " -NoNewline; write-host $varFileName -ForegroundColor Red
write-host "Parsing..."
write-host `r
# load file into memory
$varFileContents = (Get-Content -literalpath "$varFile" | Sort-Object)
start-sleep 3
clear

# check the file --------------------------------------------------------------------------------------------------------------------------

# reprise header text
write-host "Datto RMM $varProductName | Output Summary Tool"
write-host "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-" -ForegroundColor DarkGray

# verify logfile contents
$varRecordQty = @($varFileContents | select-string -Pattern '0x65FC').count

# if we find no new-style records, throw it!
if ($varRecordQty -eq 0) {
    write-host "`r"
    write-host "Error 101: File authenticity checks failed." -ForegroundColor Red
    write-host "Several things can cause this:"
    write-host " - Wrong file given (Did you mean to provide `"$varFileName`"?)"
    write-host " - File was edited before submission (please supply only unedited data)"
    write-host " - Job data was from an old version of the $varProductName Component (please re-scan using the latest)"
    write-host `r
    write-host "This file cannot be parsed; execution cannot continue."
    write-host "Please try again with a different file. Press any key to exit."
    
    #flash the 'error 101' text while waiting for input -- credit @seagull if you copy this
    while ($true) {
        $Host.UI.RawUI.FlushInputBuffer()
        start-sleep -Seconds 1
        [Console]::SetCursorPosition(0,3)
        write-host "         : File authenticity checks failed." -ForegroundColor Red
        [Console]::SetCursorPosition(0,11)
        start-sleep -Seconds 1
        [Console]::SetCursorPosition(0,3)
        write-host "Error 101: File authenticity checks failed." -ForegroundColor Red
        [Console]::SetCursorPosition(0,11)
        #check the input buffer for any key with a scan code and throw script upon detection
        if ($Host.UI.RawUI.KeyAvailable -and $Host.UI.RawUI.ReadKey("IncludeKeyUp,NoEcho").virtualKeyCode -match '^[1-9][0-9]') {
            exit
        }
    }
}

# fss tools -------------------------------------------------------------------------------------------------------------------------------

# analysis BEGIN !!
write-host "= " -NoNewline; write-host "Overview" -ForegroundColor White
write-host "Results File................] " -NoNewline -ForegroundColor Gray; write-host $varFileName -ForegroundColor Red
write-host "Records Loaded..............] " -nonewline -ForegroundColor Gray; write-host "$varRecordQty" -ForegroundColor Cyan
write-host `r

# fss programs
write-host "= " -NoNewline; write-host "File-Sync/Share Suites" -ForegroundColor White

write-host "Accellion KiteWorks.........] " -ForegroundColor Gray #line 7?
write-host "Axcient Anchor..............] " -ForegroundColor Gray
write-host "Box Drive...................] " -ForegroundColor Gray
write-host "Box Sync....................] " -ForegroundColor Gray
write-host "Citrix ShareFile............] " -ForegroundColor Gray
write-host "Dropbox.....................] " -ForegroundColor Gray
write-host "Egnyte Desktop/Storage Sync.] " -ForegroundColor Gray
write-host "Google Drive/File Stream....] " -ForegroundColor Gray
write-host "Livedrive...................] " -ForegroundColor Gray
write-host "Mozy Backup.................] " -ForegroundColor Gray
write-host "Syncplicity.................] " -ForegroundColor Gray #line 17

$arrFSSTools=@('Accellion KiteWorks','Axcient Anchor','Box Drive','Box Sync','Citrix ShareFile','Dropbox','Egnyte Desktop/Sync','Google Drive/File Stream','Livedrive','Mozy Backup','Syncplicity')
softwareLoop $arrFSSTools 7

# onedrive gets special treatment
write-host `r
$varOneDrive=@($varFileContents | Select-String -Pattern 'OneDrive').Count
if ($varOneDrive -gt 0) {
    write-host ": OneDrive is enabled on $varOneDrive/$varRecordQty systems, but is not necessarily in use."
}

# backup tools ----------------------------------------------------------------------------------------------------------------------------

write-host `r
write-host "= " -NoNewline; write-host "Backup Suites" -ForegroundColor White

write-host "Acronis Backup..............] " -ForegroundColor Gray #line 20
write-host "Ahsay OBM/ACP...............] " -ForegroundColor Gray
write-host "ArcServe....................] " -ForegroundColor Gray
write-host "Autotask Endpoint Backup....] " -ForegroundColor Gray
write-host "Autotask Workplace..........] " -ForegroundColor Gray
write-host "Backup Exec.................] " -ForegroundColor Gray
write-host "BackupAssist................] " -ForegroundColor Gray
write-host "Barracuda Backup Agent......] " -ForegroundColor Gray
write-host "COMODO Backup...............] " -ForegroundColor Gray
write-host "Carbonite...................] " -ForegroundColor Gray
write-host "CloudBerry Backup...........] " -ForegroundColor Gray
write-host "Datto Backup Agent..........] " -ForegroundColor Gray
write-host "Datto File Protection.......] " -ForegroundColor Gray
write-host "Datto Workplace.............] " -ForegroundColor Gray
write-host "EaseUS Backup...............] " -ForegroundColor Gray
write-host "Handy Backup................] " -ForegroundColor Gray
write-host "Rackspace Cloud Backup......] " -ForegroundColor Gray
write-host "ShadowProtect...............] " -ForegroundColor Gray
write-host "StorageCraft................] " -ForegroundColor Gray
write-host "Veeam.......................] " -ForegroundColor Gray #line 39

$arrBackups=@('Acronis Backup','Ahsay OBM/ACB','Arcserve','Autotask Endpoint Backup','Autotask Workplace','Backup Exec','BackupAssist','Barracuda Backup Agent','COMODO BackUp','Carbonite','CloudBerry Backup','Datto BCDR','Datto File Protection','Datto Workplace','EaseUS Backup','Handy Backup','Rackspace Cloud Backup','ShadowProtect','StorageCraft','Veeam')
softwareLoop $arrBackups 22

# check for file types --------------------------------------------------------------------------------------------------------------------

[int]$script:varTotalOffice9703=0
[int]$script:varTotalOffice0716=0
[int]$script:varTotalMedia=0
[int]$script:varTotalOther=0
fileloop "Office 1997-2003" "Office9703"
fileloop "Office 2007-version 16" "Office0716"
fileloop "Media" "Media"
fileloop "Other" "Other"

write-host `r
write-host "= " -NoNewline; write-host "File Types" -ForegroundColor White
write-host "Office 1997-2003 Files......] " -ForegroundColor Gray -nonewline; write-host $varTotalOffice9703 -foregroundcolor yellow #line 44
write-host "Office 2007-ver. 16 Files...] " -ForegroundColor Gray -nonewline; write-host $varTotalOffice0716 -foregroundcolor yellow
write-host "Media Files.................] " -ForegroundColor Gray -nonewline; write-host $varTotalMedia -foregroundcolor yellow
write-host "Other Files.................] " -ForegroundColor Gray -nonewline; write-host $varTotalOther -foregroundcolor yellow

# note what the file types actually are
[Console]::SetCursorPosition(37,44)
write-host "[ DOC XLS PPT PPS CSV PUB" -ForegroundColor DarkGray
[Console]::SetCursorPosition(37,45)
write-host "[ DOCX XLSX PPTX PPSX PPTM SLDX CSVX ACCDB" -ForegroundColor DarkGray
[Console]::SetCursorPosition(37,46)
write-host "[ JPG MP4 AVI MOV MKV MPG FLV WMV MP3 FLAC" -ForegroundColor DarkGray
[Console]::SetCursorPosition(37,47)
write-host "[ TXT ZIP RAR 7z GZ BAT PDF PS1" -ForegroundColor DarkGray
[Console]::SetCursorPosition(0,48)

# outlook time ----------------------------------------------------------------------------------------------------------------------------

write-host `r
write-host "= " -NoNewline; write-host "Outlook Mailbox Sizes" -ForegroundColor White

$varOutlook=$varFileContents | Select-String -Pattern 'consumed'
if (@($varOutlook.count) -eq 0) {
    write-host ": No Microsoft Outlook Mailbox files were found on the devices provided."
} else {
    [int]$varOutlookSize=0
    foreach ($iteration in $varOutlook) {
        $iteration=($iteration -as [string])
        $varOutlookSize+=($iteration -split 'of')[1] -replace "[^0-9]"
    }
    write-host ": Microsoft Outlook Mailboxes consumed a total of $varOutlookSize MB across $varRecordQty devices."
}

# output logging data ---------------------------------------------------------------------------------------------------------------------

write-host `r
write-host "+ Analysis completed at $(get-date)."
$varOutputFile=(([int][double]::Parse((Get-Date -UFormat %s))) -as [string]) + "-$varProductName Output.rtf"
get-ConsoleAsRTF | out-file $varOutputFile -Encoding ascii
write-host "  Logging data saved as $varOutputFile."
Write-Host `r
write-host ": Press any key to exit."
cmd /c pause 2>&1>$null